@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@
  * abstraction of the ALSA calls; it is merely a container for these
  * variables. */
 
-struct alsa_pcm_t {
+struct alsa_pcm {
     snd_pcm_t *pcm;
 
     struct pollfd *pe;
@@ -41,8 +41,8 @@ struct alsa_pcm_t {
 };
 
 
-struct alsa_t {
-    struct alsa_pcm_t capture, playback;
+struct alsa {
+    struct alsa_pcm capture, playback;
 };
 
 
@@ -52,7 +52,7 @@ static void alsa_error(const char *msg, int r)
 }
 
 
-static int pcm_open(struct alsa_pcm_t *alsa, const char *device_name,
+static int pcm_open(struct alsa_pcm *alsa, const char *device_name,
                     snd_pcm_stream_t stream, int rate, int buffer_time)
 {
     int r, dir;
@@ -154,7 +154,7 @@ static int pcm_open(struct alsa_pcm_t *alsa, const char *device_name,
 }
 
 
-static void pcm_close(struct alsa_pcm_t *alsa)
+static void pcm_close(struct alsa_pcm *alsa)
 {
     if (snd_pcm_close(alsa->pcm) < 0)
         abort();
@@ -162,7 +162,7 @@ static void pcm_close(struct alsa_pcm_t *alsa)
 }
 
 
-static ssize_t pcm_pollfds(struct alsa_pcm_t *alsa, struct pollfd *pe,
+static ssize_t pcm_pollfds(struct alsa_pcm *alsa, struct pollfd *pe,
 			   size_t z)
 {
     int r, count;
@@ -187,7 +187,7 @@ static ssize_t pcm_pollfds(struct alsa_pcm_t *alsa, struct pollfd *pe,
 }
 
 
-static int pcm_revents(struct alsa_pcm_t *alsa, unsigned short *revents) {
+static int pcm_revents(struct alsa_pcm *alsa, unsigned short *revents) {
     int r;
 
     r = snd_pcm_poll_descriptors_revents(alsa->pcm, alsa->pe, alsa->pe_count,
@@ -206,7 +206,7 @@ static int pcm_revents(struct alsa_pcm_t *alsa, unsigned short *revents) {
 
 static void start(struct device *dv)
 {
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     if (snd_pcm_start(alsa->capture.pcm) < 0)
         abort();
@@ -219,7 +219,7 @@ static void start(struct device *dv)
 static ssize_t pollfds(struct device *dv, struct pollfd *pe, size_t z)
 {
     int total, r;
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     total = 0;
 
@@ -247,7 +247,7 @@ static ssize_t pollfds(struct device *dv, struct pollfd *pe, size_t z)
 static int playback(struct device *dv)
 {
     int r;
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     device_collect(dv, alsa->playback.buf, alsa->playback.period);
 
@@ -271,7 +271,7 @@ static int playback(struct device *dv)
 static int capture(struct device *dv)
 {
     int r;
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     r = snd_pcm_readi(alsa->capture.pcm, alsa->capture.buf,
                       alsa->capture.period);
@@ -296,7 +296,7 @@ static int handle(struct device *dv)
 {
     int r;
     unsigned short revents;
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     /* Check input buffer for timecode capture */
     
@@ -365,7 +365,7 @@ static int handle(struct device *dv)
 
 static unsigned int sample_rate(struct device *dv)
 {
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     return alsa->capture.rate;
 }
@@ -375,7 +375,7 @@ static unsigned int sample_rate(struct device *dv)
 
 static void clear(struct device *dv)
 {
-    struct alsa_t *alsa = (struct alsa_t*)dv->local;
+    struct alsa *alsa = (struct alsa*)dv->local;
 
     pcm_close(&alsa->capture);
     pcm_close(&alsa->playback);
@@ -397,9 +397,9 @@ static struct device_ops alsa_ops = {
 int alsa_init(struct device *dv, const char *device_name,
               int rate, int buffer_time)
 {
-    struct alsa_t *alsa;
+    struct alsa *alsa;
 
-    alsa = malloc(sizeof(struct alsa_t));
+    alsa = malloc(sizeof(struct alsa));
     if (!alsa) {
         perror("malloc");
         return -1;
