@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,21 +21,29 @@
 #define DECK_H
 
 #include "device.h"
+#include "listing.h"
 #include "player.h"
-#include "rig.h"
 #include "realtime.h"
 #include "timecoder.h"
-#include "track.h"
 
-struct deck_t {
+struct deck {
     struct device device;
-    struct timecoder_t timecoder;
-    struct track_t track;
+    struct timecoder timecoder;
+    const char *importer;
+    bool protect;
 
-    struct player_t player;
+    struct player player;
+    const struct record *record;
 };
 
-int deck_init(struct deck_t *deck, struct rt_t *rt, struct rig_t *rig);
-void deck_clear(struct deck_t *deck);
+int deck_init(struct deck *deck, struct rt *rt);
+void deck_clear(struct deck *deck);
+
+bool deck_is_locked(const struct deck *deck);
+
+void deck_load(struct deck *deck, struct record *record);
+
+void deck_recue(struct deck *deck);
+void deck_clone(struct deck *deck, const struct deck *from);
 
 #endif
