@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,12 +24,12 @@
 #include "player.h"
 #include "timecoder.h"
 
-void device_connect_timecoder(struct device *dv, struct timecoder_t *tc)
+void device_connect_timecoder(struct device *dv, struct timecoder *tc)
 {
     dv->timecoder = tc;
 }
 
-void device_connect_player(struct device *dv, struct player_t *pl)
+void device_connect_player(struct device *dv, struct player *pl)
 {
     dv->player = pl;
 }
