@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,8 +29,8 @@ struct device {
     void *local;
     struct device_ops *ops;
 
-    struct timecoder_t *timecoder;
-    struct player_t *player;
+    struct timecoder *timecoder;
+    struct player *player;
 };
 
 struct device_ops {
@@ -44,8 +44,8 @@ struct device_ops {
     void (*clear)(struct device *dv);
 };
 
-void device_connect_timecoder(struct device *dv, struct timecoder_t *tc);
-void device_connect_player(struct device *dv, struct player_t *pl);
+void device_connect_timecoder(struct device *dv, struct timecoder *tc);
+void device_connect_player(struct device *dv, struct player *pl);
 
 unsigned int device_sample_rate(struct device *dv);
 
