@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,7 @@
 #include <errno.h>
 #include <math.h>
 #include <pthread.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,7 +33,6 @@
 #include <SDL_ttf.h>
 
 #include "interface.h"
-#include "listing.h"
 #include "player.h"
 #include "rig.h"
 #include "selector.h"
@@ -144,7 +144,12 @@ static SDL_Color background_col = {0, 0, 0, 255},
 
 static int spinner_angle[SPINNER_SIZE * SPINNER_SIZE];
 
-struct rect_t {
+static pthread_t ph;
+static struct deck *deck;
+static size_t ndeck;
+static struct selector selector;
+
+struct rect {
     signed short x, y, w, h;
 };
 
@@ -155,8 +160,8 @@ struct rect_t {
  * Post: upper and lower are set
  */
 
-static void split_top(const struct rect_t *source, struct rect_t *upper,
-                      struct rect_t *lower, int v, int space)
+static void split_top(const struct rect *source, struct rect *upper,
+                      struct rect *lower, int v, int space)
 {
     int u;
 
@@ -181,8 +186,8 @@ static void split_top(const struct rect_t *source, struct rect_t *upper,
  * As above, x pixels from the bottom
  */
 
-static void split_bottom(const struct rect_t *source, struct rect_t *upper,
-                         struct rect_t *lower, int v, int space)
+static void split_bottom(const struct rect *source, struct rect *upper,
+                         struct rect *lower, int v, int space)
 {
     split_top(source, upper, lower, source->h - v - space, space);
 }
@@ -191,8 +196,8 @@ static void split_bottom(const struct rect_t *source, struct rect_t *upper,
  * As above, v pixels from the left
  */
 
-static void split_left(const struct rect_t *source, struct rect_t *left,
-                       struct rect_t *right, int v, int space)
+static void split_left(const struct rect *source, struct rect *left,
+                       struct rect *right, int v, int space)
 {
     int u;
 
@@ -217,8 +222,8 @@ static void split_left(const struct rect_t *source, struct rect_t *left,
  * As above, v pixels from the right
  */
 
-static void split_right(const struct rect_t *source, struct rect_t *left,
-                        struct rect_t *right, int v, int space)
+static void split_right(const struct rect *source, struct rect *left,
+                        struct rect *right, int v, int space)
 {
     split_left(source, left, right, source->w - v - space, space);
 }
@@ -229,13 +234,14 @@ static void split_right(const struct rect_t *source, struct rect_t *left,
 
 static void time_to_clock(char *buf, char *deci, int t)
 {
-    int minutes, seconds, frac, neg;
+    int minutes, seconds, frac;
+    bool neg;
 
     if (t < 0) {
         t = abs(t);
-        neg = 1;
+        neg = true;
     } else
-        neg = 0;
+        neg = false;
 
     minutes = (t / 60 / 1000) % (60*60);
     seconds = (t / 1000) % 60;
@@ -409,7 +415,7 @@ static Uint32 palette(SDL_Surface *sf, SDL_Color *col)
  * Return: width of text drawn
  */
 
-static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
+static int draw_text(SDL_Surface *sf, const struct rect *rect,
                      const char *buf, TTF_Font *font,
                      SDL_Color fg, SDL_Color bg)
 {
@@ -429,11 +435,11 @@ static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
 
         src.x = 0;
         src.y = 0;
-        src.w = MIN(w, rendered->w);
-        src.h = MIN(h, rendered->h);
+        src.w = MIN(rect->w, rendered->w);
+        src.h = MIN(rect->h, rendered->h);
 
-        dst.x = x;
-        dst.y = y;
+        dst.x = rect->x;
+        dst.y = rect->y;
 
         SDL_BlitSurface(rendered, &src, sf, &dst);
         SDL_FreeSurface(rendered);
@@ -441,19 +447,19 @@ static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
 
     /* Complete the remaining space with a blank rectangle */
 
-    if (src.w < w) {
-        fill.x = x + src.w;
-        fill.y = y;
-        fill.w = w - src.w;
-        fill.h = h;
+    if (src.w < rect->w) {
+        fill.x = rect->x + src.w;
+        fill.y = rect->y;
+        fill.w = rect->w - src.w;
+        fill.h = rect->h;
         SDL_FillRect(sf, &fill, palette(sf, &bg));
     }
 
-    if (src.h < h) {
-        fill.x = x;
-        fill.y = y + src.h;
+    if (src.h < rect->h) {
+        fill.x = rect->x;
+        fill.y = rect->y + src.h;
         fill.w = src.w; /* the x-fill rectangle does the corner */
-        fill.h = h - src.h;
+        fill.h = rect->h - src.h;
         SDL_FillRect(sf, &fill, palette(sf, &bg));
     }
 
@@ -461,51 +467,53 @@ static int draw_font(SDL_Surface *sf, int x, int y, int w, int h,
 }
 
 /*
- * Draw text at the given rectangle
- *
- * Return: width of text drawn
+ * Draw a coloured rectangle
  */
 
-static int draw_font_rect(SDL_Surface *surface, const struct rect_t *rect,
-                          const char *buf, TTF_Font *font,
-                          SDL_Color fg, SDL_Color bg)
+static void draw_rect(SDL_Surface *surface, const struct rect *rect,
+                      SDL_Color col)
 {
-    return draw_font(surface, rect->x, rect->y, rect->w, rect->h,
-                     buf, font, fg, bg);
+    SDL_Rect b;
+
+    b.x = rect->x;
+    b.y = rect->y;
+    b.w = rect->w;
+    b.h = rect->h;
+    SDL_FillRect(surface, &b, palette(surface, &col));
 }
 
 /*
- * Draw the display of artist name and track name
+ * Draw the record information in the deck
  */
 
-static void draw_track_summary(SDL_Surface *surface, const struct rect_t *rect,
-                               struct track_t *track)
+static void draw_record(SDL_Surface *surface, const struct rect *rect,
+                        const struct record *record)
 {
-    struct rect_t top, bottom;
+    struct rect top, bottom;
 
     split_top(rect, &top, &bottom, FONT_SPACE, 0);
 
-    draw_font_rect(surface, &top, track->artist,
-                   font, text_col, background_col);
-    draw_font_rect(surface, &bottom, track->title,
-                   em_font, text_col, background_col);
+    draw_text(surface, &top, record->artist,
+              font, text_col, background_col);
+    draw_text(surface, &bottom, record->title,
+              em_font, text_col, background_col);
 }
 
 /*
  * Draw a single time in milliseconds in hours:minutes.seconds format
  */
 
-static void draw_clock(SDL_Surface *surface, const struct rect_t *rect, int t,
+static void draw_clock(SDL_Surface *surface, const struct rect *rect, int t,
                        SDL_Color col)
 {
     char hms[8], deci[8];
     short int v;
     int offset;
-    struct rect_t sr;
+    struct rect sr;
 
     time_to_clock(hms, deci, t);
 
-    v = draw_font_rect(surface, rect, hms, clock_font, col, background_col);
+    v = draw_text(surface, rect, hms, clock_font, col, background_col);
 
     offset = CLOCK_FONT_SIZE - DECI_FONT_SIZE * 1.04;
 
@@ -514,15 +522,15 @@ static void draw_clock(SDL_Surface *surface, const struct rect_t *rect, int t,
     sr.w = rect->w - v;
     sr.h = rect->h - offset;
 
-    draw_font_rect(surface, &sr, deci, deci_font, col, background_col);
+    draw_text(surface, &sr, deci, deci_font, col, background_col);
 }
 
 /*
  * Draw the visual monitor of the input audio to the timecoder
  */
 
-static void draw_scope(SDL_Surface *surface, const struct rect_t *rect,
-                       struct timecoder_t *tc)
+static void draw_scope(SDL_Surface *surface, const struct rect *rect,
+                       struct timecoder *tc)
 {
     int r, c, v, mid;
     Uint8 *p;
@@ -554,22 +562,24 @@ static void draw_scope(SDL_Surface *surface, const struct rect_t *rect,
  * matches the physical rotation of the vinyl record.
  */
 
-static void draw_spinner(SDL_Surface *surface, const struct rect_t *rect,
-                         struct player_t *pl)
+static void draw_spinner(SDL_Surface *surface, const struct rect *rect,
+                         struct player *pl)
 {
     int x, y, r, c, rangle, pangle;
-    double position, rps;
+    double elapsed, remain, rps;
     Uint8 *rp, *p;
     SDL_Color col;
 
     x = rect->x;
     y = rect->y;
 
-    position = pl->position - pl->offset;
-    rps = timecoder_revs_per_sec(pl->timecoder);
-    rangle = (int)(pl->position * 1024 * rps) % 1024;
+    elapsed = player_get_elapsed(pl);
+    remain = player_get_remain(pl);
 
-    if (position < 0 || position >= (double)pl->track->length / pl->track->rate)
+    rps = timecoder_revs_per_sec(pl->timecoder);
+    rangle = (int)(player_get_position(pl) * 1024 * rps) % 1024;
+
+    if (elapsed < 0 || remain < 0)
         col = warn_col;
     else
         col = ok_col;
@@ -608,42 +618,39 @@ static void draw_spinner(SDL_Surface *surface, const struct rect_t *rect,
  * Draw the clocks which show time elapsed and time remaining
  */
 
-static void draw_deck_clocks(SDL_Surface *surface, const struct rect_t *rect,
-                             struct player_t *pl)
+static void draw_deck_clocks(SDL_Surface *surface, const struct rect *rect,
+                             struct player *pl, struct track *track)
 {
     int elapse, remain;
-    float pos;
-    struct rect_t upper, lower;
+    struct rect upper, lower;
     SDL_Color col;
 
     split_top(rect, &upper, &lower, CLOCK_FONT_SIZE, 0);
 
-    pos = pl->position - pl->offset;
-
-    elapse = pos * 1000;
-    remain = (pos - pl->track->length / pl->track->rate) * 1000;
+    elapse = player_get_elapsed(pl) * 1000;
+    remain = player_get_remain(pl) * 1000;
 
     if (elapse < 0)
         col = warn_col;
-    else if (remain <= 0)
+    else if (remain > 0)
         col = ok_col;
     else
         col = text_col;
 
     draw_clock(surface, &upper, elapse, col);
 
-    if (remain > 0)
+    if (remain <= 0)
         col = warn_col;
     else
         col = text_col;
 
-    if (track_is_importing(pl->track)) {
+    if (track_is_importing(track)) {
         col.r >>= 2;
         col.g >>= 2;
         col.b >>= 2;
     }
 
-    draw_clock(surface, &lower, remain, col);
+    draw_clock(surface, &lower, -remain, col);
 }
 
 /*
@@ -651,8 +658,8 @@ static void draw_deck_clocks(SDL_Surface *surface, const struct rect_t *rect,
  * of the track
  */
 
-static void draw_overview(SDL_Surface *surface, const struct rect_t *rect,
-                          struct track_t *tr, int position)
+static void draw_overview(SDL_Surface *surface, const struct rect *rect,
+                          struct track *tr, int position)
 {
     int x, y, w, h, r, c, sp, fade, bytes_per_pixel, pitch, height,
         current_position;
@@ -739,8 +746,8 @@ static void draw_overview(SDL_Surface *surface, const struct rect_t *rect,
  * 'scale'
  */
 
-static void draw_closeup(SDL_Surface *surface, const struct rect_t *rect,
-                         struct track_t *tr, int position, int scale)
+static void draw_closeup(SDL_Surface *surface, const struct rect *rect,
+                         struct track *tr, int position, int scale)
 {
     int x, y, w, h, c;
     size_t bytes_per_pixel, pitch;
@@ -810,10 +817,10 @@ static void draw_closeup(SDL_Surface *surface, const struct rect_t *rect,
  * Draw the audio meters for a deck
  */
 
-static void draw_meters(SDL_Surface *surface, const struct rect_t *rect,
-                        struct track_t *tr, int position, int scale)
+static void draw_meters(SDL_Surface *surface, const struct rect *rect,
+                        struct track *tr, int position, int scale)
 {
-    struct rect_t overview, closeup;
+    struct rect overview, closeup;
 
     split_top(rect, &overview, &closeup, OVERVIEW_HEIGHT, SPACER);
 
@@ -829,10 +836,10 @@ static void draw_meters(SDL_Surface *surface, const struct rect_t *rect,
  * Draw the current playback status -- clocks, spinner and scope
  */
 
-static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
-                          struct player_t *pl)
+static void draw_deck_top(SDL_Surface *surface, const struct rect *rect,
+                          struct player *pl, struct track *track)
 {
-    struct rect_t clocks, left, right, spinner, scope;
+    struct rect clocks, left, right, spinner, scope;
 
     split_left(rect, &clocks, &right, CLOCKS_WIDTH, SPACER);
 
@@ -840,11 +847,11 @@ static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
      * available space, just draw clocks which span the overall space */
 
     if (!pl->timecode_control || right.w < 0) {
-        draw_deck_clocks(surface, rect, pl);
+        draw_deck_clocks(surface, rect, pl, track);
         return;
     }
 
-    draw_deck_clocks(surface, &clocks, pl);
+    draw_deck_clocks(surface, &clocks, pl, track);
 
     split_right(&right, &left, &spinner, SPINNER_SIZE, SPACER);
     if (left.w < 0)
@@ -865,11 +872,12 @@ static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
  */
 
 static void draw_deck_status(SDL_Surface *surface,
-                                 const struct rect_t *rect,
-                                 struct player_t *pl)
+                             const struct rect *rect,
+                             const struct deck *deck)
 {
     char buf[128], *c;
     int tc;
+    const struct player *pl = &deck->player;
 
     c = buf;
 
@@ -882,62 +890,64 @@ static void draw_deck_status(SDL_Surface *surface,
         c += sprintf(c, "        ");
     }
 
-    sprintf(c, "pitch:%+0.2f (sync %0.2f %+.5fs = %+0.2f)  %s",
+    sprintf(c, "pitch:%+0.2f (sync %0.2f %+.5fs = %+0.2f)  %s%s",
             pl->pitch,
             pl->sync_pitch,
             pl->last_difference,
             pl->pitch * pl->sync_pitch,
-            pl->recalibrate ? "RCAL  " : "");
+            pl->recalibrate ? "RCAL  " : "",
+            deck_is_locked(deck) ? "LOCK  " : "");
 
-    draw_font_rect(surface, rect, buf, detail_font,
-                   detail_col, background_col);
+    draw_text(surface, rect, buf, detail_font, detail_col, background_col);
 }
 
 /*
  * Draw a single deck
  */
 
-static void draw_deck(SDL_Surface *surface, const struct rect_t *rect,
-                      struct deck_t *deck, int meter_scale)
+static void draw_deck(SDL_Surface *surface, const struct rect *rect,
+                      struct deck *deck, int meter_scale)
 {
     int position;
-    struct rect_t track, top, meters, status, rest, lower;
-    struct player_t *pl;
+    struct rect track, top, meters, status, rest, lower;
+    struct player *pl;
+    struct track *t;
 
     pl = &deck->player;
+    t = pl->track;
 
-    position = (pl->position - pl->offset) * pl->track->rate;
+    position = player_get_elapsed(pl) * t->rate;
 
     split_top(rect, &track, &rest, FONT_SPACE * 2, 0);
     if (rest.h < 160)
         rest = *rect;
     else
-        draw_track_summary(surface, &track, pl->track);
+        draw_record(surface, &track, deck->record);
 
     split_top(&rest, &top, &lower, CLOCK_FONT_SIZE * 2, SPACER);
     if (lower.h < 64)
         lower = rest;
     else
-        draw_deck_top(surface, &top, pl);
+        draw_deck_top(surface, &top, pl, t);
 
     split_bottom(&lower, &meters, &status, FONT_SPACE, SPACER);
     if (meters.h < 64)
         meters = lower;
     else
-        draw_deck_status(surface, &status, pl);
+        draw_deck_status(surface, &status, deck);
 
-    draw_meters(surface, &meters, pl->track, position, meter_scale);
+    draw_meters(surface, &meters, t, position, meter_scale);
 }
 
 /*
  * Draw all the decks in the system left to right
  */
 
-static void draw_decks(SDL_Surface *surface, const struct rect_t *rect,
-                       struct deck_t deck[], size_t ndecks, int meter_scale)
+static void draw_decks(SDL_Surface *surface, const struct rect *rect,
+                       struct deck deck[], size_t ndecks, int meter_scale)
 {
     int d, deck_width;
-    struct rect_t single;
+    struct rect single;
 
     deck_width = (rect->w - BORDER * (ndecks - 1)) / ndecks;
 
@@ -954,24 +964,24 @@ static void draw_decks(SDL_Surface *surface, const struct rect_t *rect,
  * Draw the status bar
  */
 
-static void draw_status(SDL_Surface *sf, const struct rect_t *rect,
+static void draw_status(SDL_Surface *sf, const struct rect *rect,
                         const char *text)
 {
-    draw_font_rect(sf, rect, text, detail_font, detail_col, background_col);
+    draw_text(sf, rect, text, detail_font, detail_col, background_col);
 }
 
 /*
  * Draw the search field which the user types into
  */
 
-static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
-                        struct selector_t *sel)
+static void draw_search(SDL_Surface *surface, const struct rect *rect,
+                        struct selector *sel)
 {
     int s;
     const char *buf;
     char cm[32];
     SDL_Rect cursor;
-    struct rect_t rtext;
+    struct rect rtext;
 
     split_left(rect, NULL, &rtext, SCROLLBAR_SIZE, SPACER);
 
@@ -980,7 +990,7 @@ static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
     else
         buf = NULL;
 
-    s = draw_font_rect(surface, &rtext, buf, font, text_col, background_col);
+    s = draw_text(surface, &rtext, buf, font, text_col, background_col);
 
     cursor.x = rtext.x + s;
     cursor.y = rtext.y;
@@ -999,7 +1009,7 @@ static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
     rtext.x += s + CURSOR_WIDTH + SPACER;
     rtext.w -= s + CURSOR_WIDTH + SPACER;
 
-    draw_font_rect(surface, &rtext, cm, em_font, detail_col, background_col);
+    draw_text(surface, &rtext, cm, em_font, detail_col, background_col);
 }
 
 /*
@@ -1007,8 +1017,8 @@ static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
  * given number of entries
  */
 
-static void draw_scroll_bar(SDL_Surface *surface, const struct rect_t *rect,
-                            struct scroll_t *scroll)
+static void draw_scroll_bar(SDL_Surface *surface, const struct rect *rect,
+                            const struct scroll *scroll)
 {
     SDL_Rect box;
     SDL_Color bg;
@@ -1039,59 +1049,42 @@ static void draw_scroll_bar(SDL_Surface *surface, const struct rect_t *rect,
  * Return: the number of lines which fit on the display.
  */
 
-static void draw_crates(SDL_Surface *surface, const struct rect_t *rect,
-                        struct selector_t *sel)
+static void draw_crates(SDL_Surface *surface, const struct rect *rect,
+                        const struct library *library,
+                        const struct scroll *scroll)
 {
-    int x, y, w, h, r, ox, n;
-    struct rect_t rs;
-    SDL_Rect box;
-    SDL_Color col_bg, col_text;
+    size_t n;
+    struct rect left, bottom;
 
-    x = rect->x;
-    y = rect->y;
-    w = rect->w;
-    h = rect->h;
-    ox = x;
+    split_left(rect, &left, &bottom, SCROLLBAR_SIZE, SPACER);
+    draw_scroll_bar(surface, &left, scroll);
 
-    x += SCROLLBAR_SIZE + SPACER;
-    w -= SCROLLBAR_SIZE + SPACER;
+    n = scroll->offset;
 
-    for (n = 0; n + sel->crates.offset < sel->library->crates; n++) {
+    for (;;) {
+        SDL_Color col_text, col_bg;
+        struct rect top;
+        const struct crate *crate;
 
-        if ((n + 1) * FONT_SPACE > h)
+        if (n >= library->crates)
             break;
 
-        r = y + n * FONT_SPACE;
+        if (bottom.h < FONT_SPACE)
+            break;
 
-        if (sel->library->crate[n + sel->crates.offset]->is_fixed)
-            col_text = detail_col;
-        else
-            col_text = text_col;
+        split_top(&bottom, &top, &bottom, FONT_SPACE, 0);
 
-        if (n + sel->crates.offset == sel->crates.selected)
-            col_bg = selected_col;
-        else
-            col_bg = background_col;
+        crate = library->crate[n];
 
-        draw_font(surface, x, r, w, FONT_SPACE,
-                  sel->library->crate[n + sel->crates.offset]->name, font,
-                  col_text, col_bg);
+        col_text = crate->is_fixed ? detail_col: text_col;
+        col_bg = (n == scroll->selected) ? selected_col : background_col;
+
+        draw_text(surface, &top, crate->name, font, col_text, col_bg);
+
+        n++;
     }
 
-    /* Blank any remaining space */
-
-    box.x = x;
-    box.y = y + n * FONT_SPACE;
-    box.w = w;
-    box.h = h - (n * FONT_SPACE);
-
-    SDL_FillRect(surface, &box, palette(surface, &background_col));
-
-    rs.x = ox;
-    rs.y = y;
-    rs.w = SCROLLBAR_SIZE;
-    rs.h = h;
-    draw_scroll_bar(surface, &rs, &sel->crates);
+    draw_rect(surface, &bottom, background_col);
 }
 
 /*
@@ -1101,67 +1094,50 @@ static void draw_crates(SDL_Surface *surface, const struct rect_t *rect,
  * Return: the number of lines which fit on the display
  */
 
-static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
-                         struct selector_t *sel)
+static void draw_listing(SDL_Surface *surface, const struct rect *rect,
+                         const struct listing *listing,
+                         const struct scroll *scroll)
 {
-    int x, y, w, h, n, r, ox;
-    struct rect_t rs;
-    struct record_t *re;
-    SDL_Rect box;
-    SDL_Color col;
+    size_t n;
+    struct rect left, bottom;
 
-    x = rect->x;
-    y = rect->y;
-    w = rect->w;
-    h = rect->h;
+    split_left(rect, &left, &bottom, SCROLLBAR_SIZE, SPACER);
+    draw_scroll_bar(surface, &left, scroll);
 
-    ox = x;
+    n = scroll->offset;
 
-    x += SCROLLBAR_SIZE + SPACER;
-    w -= SCROLLBAR_SIZE + SPACER;
+    for (;;) {
+        SDL_Color col;
+        struct rect top, left, right;
+        const struct record *record;
 
-    for (n = 0; n + sel->records.offset < sel->view_listing->entries; n++) {
-        re = sel->view_listing->record[n + sel->records.offset];
-
-        if ((n + 1) * FONT_SPACE > h)
+        if (n >= listing->entries)
             break;
 
-        r = y + n * FONT_SPACE;
+        if (bottom.h < FONT_SPACE)
+            break;
 
-        if (n + sel->records.offset == sel->records.selected)
+        split_top(&bottom, &top, &bottom, FONT_SPACE, 0);
+
+        record = listing->record[n];
+
+        if (n == scroll->selected) {
             col = selected_col;
-        else
+        } else {
             col = background_col;
+        }
 
-        draw_font(surface, x, r, RESULTS_ARTIST_WIDTH, FONT_SPACE,
-                  re->artist, font, text_col, col);
+        split_left(&top, &left, &right, RESULTS_ARTIST_WIDTH, 0);
+        draw_text(surface, &left, record->artist, font, text_col, col);
 
-        box.x = x + RESULTS_ARTIST_WIDTH;
-        box.y = r;
-        box.w = SPACER;
-        box.h = FONT_SPACE;
+        split_left(&right, &left, &right, SPACER, 0);
+        draw_rect(surface, &left, col);
+        draw_text(surface, &right, record->title, font, text_col, col);
 
-        SDL_FillRect(surface, &box, palette(surface, &col));
-
-        draw_font(surface, x + RESULTS_ARTIST_WIDTH + SPACER, r,
-                  w - RESULTS_ARTIST_WIDTH - SPACER, FONT_SPACE,
-                  re->title, em_font, text_col, col);
+        n++;
     }
 
-    /* Blank any remaining space */
-
-    box.x = x;
-    box.y = y + n * FONT_SPACE;
-    box.w = w;
-    box.h = h - (n * FONT_SPACE);
-
-    SDL_FillRect(surface, &box, palette(surface, &background_col));
-
-    rs.x = ox;
-    rs.y = y;
-    rs.w = SCROLLBAR_SIZE;
-    rs.h = h;
-    draw_scroll_bar(surface, &rs, &sel->records);
+    draw_rect(surface, &bottom, background_col);
 }
 
 /*
@@ -1169,11 +1145,11 @@ static void draw_records(SDL_Surface *surface, const struct rect_t *rect,
  * results
  */
 
-static void draw_library(SDL_Surface *surface, const struct rect_t *rect,
-                         struct selector_t *sel)
+static void draw_library(SDL_Surface *surface, const struct rect *rect,
+                         struct selector *sel)
 {
     unsigned int lines;
-    struct rect_t rsearch, rlists, rcrates, rrecords;
+    struct rect rsearch, rlists, rcrates, rrecords;
 
     split_top(rect, &rsearch, &rlists, SEARCH_HEIGHT, SPACER);
     draw_search(surface, &rsearch, sel);
@@ -1183,40 +1159,22 @@ static void draw_library(SDL_Surface *surface, const struct rect_t *rect,
 
     split_left(&rlists, &rcrates, &rrecords, (rlists.w / 4), SPACER);
     if (rcrates.w > LIBRARY_MIN_WIDTH) {
-        draw_records(surface, &rrecords, sel);
-        draw_crates(surface, &rcrates, sel);
+        draw_listing(surface, &rrecords, sel->view_listing, &sel->records);
+        draw_crates(surface, &rcrates, sel->library, &sel->crates);
     } else {
-        draw_records(surface, rect, sel);
+        draw_listing(surface, rect, sel->view_listing, &sel->records);
     }
 }
 
-/*
- * Initiate the process of loading a record from the library into a
- * track in memory
- */
-
-static void do_loading(struct interface_t *interface,
-                       struct track_t *track, struct record_t *record)
-{
-    fprintf(stderr, "Loading '%s'.\n", record->pathname);
-
-    if (track_import(track, record->pathname) == -1)
-        return;
-
-    track->artist = record->artist;
-    track->title = record->title;
-
-    (void)rig_awaken(interface->rig);
-}
-
 /*
  * Handle a single key event
  *
  * Return: true if the selector needs to be redrawn, otherwise false
  */
 
-static bool handle_key(struct interface_t *in, struct selector_t *sel,
-                       int *meter_scale, SDLKey key, SDLMod mod)
+static bool handle_key(struct deck deck[], size_t ndeck,
+                       struct selector *sel, int *meter_scale,
+                       SDLKey key, SDLMod mod)
 {
     if (key >= SDLK_a && key <= SDLK_z) {
         selector_search_refine(sel, (key - SDLK_a) + 'a');
@@ -1298,32 +1256,32 @@ static bool handle_key(struct interface_t *in, struct selector_t *sel,
 
         d = (key - SDLK_F1) / 4;
 
-        if (d < in->ndeck) {
+        if (d < ndeck) {
             int func;
-            struct deck_t *deck;
-            struct player_t *pl;
-            struct record_t *re;
-            struct timecoder_t *tc;
+            struct deck *de;
+            struct player *pl;
+            struct record *re;
+            struct timecoder *tc;
 
             func = (key - SDLK_F1) % 4;
 
-            deck = &in->deck[d];
-            pl = &deck->player;
-            tc = &deck->timecoder;
+            de = &deck[d];
+            pl = &de->player;
+            tc = &de->timecoder;
 
             if (mod & KMOD_SHIFT) {
-                if (func < in->ndeck)
-                    player_set_timecoder(pl, &in->deck[func].timecoder);
+                if (func < ndeck)
+                    deck_clone(de, &deck[func]);
 
             } else switch(func) {
             case FUNC_LOAD:
                 re = selector_current(sel);
                 if (re != NULL)
-                    do_loading(in, &deck->track, re);
+                    deck_load(de, re);
                 break;
 
             case FUNC_RECUE:
-                player_recue(pl);
+                deck_recue(de);
                 break;
 
             case FUNC_TIMECODE:
@@ -1344,7 +1302,7 @@ static bool handle_key(struct interface_t *in, struct selector_t *sel,
  * Action on size change event on the main window
  */
 
-static SDL_Surface* set_size(int w, int h, struct rect_t *rect)
+static SDL_Surface* set_size(int w, int h, struct rect *rect)
 {
     SDL_Surface *surface;
 
@@ -1386,7 +1344,7 @@ static Uint32 ticker(Uint32 interval, void *p)
  * The SDL interface thread
  */
 
-static int interface_main(struct interface_t *in)
+static int interface_main(void)
 {
     int meter_scale, library_update, decks_update, status_update;
     const char *status = banner;
@@ -1395,7 +1353,7 @@ static int interface_main(struct interface_t *in)
     SDL_TimerID timer;
     SDL_Surface *surface;
 
-    struct rect_t rworkspace, rplayers, rlibrary, rstatus, rtmp;
+    struct rect rworkspace, rplayers, rlibrary, rstatus, rtmp;
 
     meter_scale = DEFAULT_METER_SCALE;
 
@@ -1409,13 +1367,13 @@ static int interface_main(struct interface_t *in)
 
     /* The final action is to add the timer which triggers refresh */
 
-    timer = SDL_AddTimer(REFRESH, ticker, (void*)in);
+    timer = SDL_AddTimer(REFRESH, ticker, NULL);
 
     while (SDL_WaitEvent(&event) >= 0) {
 
         switch(event.type) {
         case SDL_QUIT: /* user request to quit application; eg. window close */
-            if (rig_quit(in->rig) == -1)
+            if (rig_quit() == -1)
                 return -1;
             break;
 
@@ -1445,8 +1403,8 @@ static int interface_main(struct interface_t *in)
             break;
 
         case SDL_KEYDOWN:
-            if (handle_key(in, &in->selector, &meter_scale,
-                          event.key.keysym.sym, event.key.keysym.mod))
+            if (handle_key(deck, ndeck, &selector, &meter_scale,
+                           event.key.keysym.sym, event.key.keysym.mod))
             {
                 library_update = UPDATE_REDRAW;
             }
@@ -1476,14 +1434,14 @@ static int interface_main(struct interface_t *in)
 
         if (library_update == UPDATE_REDRAW) {
 
-            if (selector_current(&in->selector) != NULL)
-                status = selector_current(&in->selector)->pathname;
+            if (selector_current(&selector) != NULL)
+                status = selector_current(&selector)->pathname;
             else
                 status = "No search results found";
             status_update = UPDATE_REDRAW;
 
             LOCK(surface);
-            draw_library(surface, &rlibrary, &in->selector);
+            draw_library(surface, &rlibrary, &selector);
             UNLOCK(surface);
             UPDATE(surface, &rlibrary);
             library_update = UPDATE_NONE;
@@ -1504,7 +1462,7 @@ static int interface_main(struct interface_t *in)
 
         if (decks_update == UPDATE_REDRAW) {
             LOCK(surface);
-            draw_decks(surface, &rplayers, in->deck, in->ndeck, meter_scale);
+            draw_decks(surface, &rplayers, deck, ndeck, meter_scale);
             UNLOCK(surface);
             UPDATE(surface, &rplayers);
             decks_update = UPDATE_NONE;
@@ -1520,7 +1478,7 @@ static int interface_main(struct interface_t *in)
 
 static void* launch(void *p)
 {
-    interface_main(p);
+    interface_main();
     return NULL;
 }
 
@@ -1531,21 +1489,19 @@ static void* launch(void *p)
  * error
  */
 
-int interface_start(struct interface_t *in, struct deck_t deck[], size_t ndeck,
-                    struct library_t *lib, struct rig_t *rig)
+int interface_start(struct deck ldeck[], size_t lndeck, struct library *lib)
 {
     size_t n;
 
+    deck = ldeck;
+    ndeck = lndeck;
+
     for (n = 0; n < ndeck; n++) {
         if (timecoder_monitor_init(&deck[n].timecoder, SCOPE_SIZE) == -1)
             return -1;
     }
 
-    in->deck = deck;
-    in->ndeck = ndeck;
-
-    in->rig = rig;
-    selector_init(&in->selector, lib);
+    selector_init(&selector, lib);
     calculate_spinner_lookup(spinner_angle, NULL, SPINNER_SIZE);
 
     fprintf(stderr, "Initialising SDL...\n");
@@ -1569,7 +1525,7 @@ int interface_start(struct interface_t *in, struct deck_t deck[], size_t ndeck,
 
     fprintf(stderr, "Launching interface thread...\n");
 
-    if (pthread_create(&in->ph, NULL, launch, (void*)in)) {
+    if (pthread_create(&ph, NULL, launch, NULL)) {
         perror("pthread_create");
         return -1;
     }
@@ -1581,7 +1537,7 @@ int interface_start(struct interface_t *in, struct deck_t deck[], size_t ndeck,
  * Synchronise with the SDL interface and exit
  */
 
-void interface_stop(struct interface_t *in)
+void interface_stop(void)
 {
     size_t n;
     SDL_Event quit;
@@ -1591,13 +1547,13 @@ void interface_stop(struct interface_t *in)
     if (SDL_PushEvent(&quit) == -1)
         abort();
 
-    if (pthread_join(in->ph, NULL) != 0)
+    if (pthread_join(ph, NULL) != 0)
         abort();
 
-    for (n = 0; n < in->ndeck; n++)
-        timecoder_monitor_clear(&in->deck[n].timecoder);
+    for (n = 0; n < ndeck; n++)
+        timecoder_monitor_clear(&deck[n].timecoder);
 
-    selector_clear(&in->selector);
+    selector_clear(&selector);
 
     clear_fonts();
 
