@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,25 +20,10 @@
 #ifndef INTERFACE_H
 #define INTERFACE_H
 
-#include <pthread.h>
-
 #include "deck.h"
 #include "library.h"
-#include "rig.h"
-#include "selector.h"
 
-struct interface_t {
-    pthread_t ph;
-
-    size_t ndeck;
-    struct deck_t *deck;
-
-    struct rig_t *rig;
-    struct selector_t selector;
-};
-
-int interface_start(struct interface_t *in, struct deck_t deck[], size_t ndeck,
-                    struct library_t *lib, struct rig_t *rig);
-void interface_stop(struct interface_t *in);
+int interface_start(struct deck deck[], size_t ndeck, struct library *lib);
+void interface_stop();
 
 #endif
