@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,26 +22,26 @@
 
 #include <stddef.h>
 
-struct record_t {
+struct record {
     char *pathname, *artist, *title;
 };
 
 /* Listing points to records, but does not manage those pointers */
 
-struct listing_t {
-    struct record_t **record;
+struct listing {
+    struct record **record;
     size_t size, entries;
 };
 
-void listing_init(struct listing_t *ls);
-void listing_clear(struct listing_t *ls);
-void listing_blank(struct listing_t *ls);
-int listing_add(struct listing_t *li, struct record_t *lr);
-int listing_copy(const struct listing_t *src, struct listing_t *dest);
-int listing_match(struct listing_t *src, struct listing_t *dest,
+void listing_init(struct listing *ls);
+void listing_clear(struct listing *ls);
+void listing_blank(struct listing *ls);
+int listing_add(struct listing *li, struct record *lr);
+int listing_copy(const struct listing *src, struct listing *dest);
+int listing_match(struct listing *src, struct listing *dest,
 		  const char *match);
-struct record_t* listing_insert(struct listing_t *ls, struct record_t *item);
-void listing_debug(struct listing_t *ls);
-void listing_sort(struct listing_t *ls);
+struct record* listing_insert(struct listing *ls, struct record *item);
+void listing_debug(struct listing *ls);
+void listing_sort(struct listing *ls);
 
 #endif
