@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
 
 #include "device.h"
 
-int oss_init(struct device_t *dv, const char *filename, unsigned int rate,
+int oss_init(struct device *dv, const char *filename, unsigned int rate,
              unsigned short buffers, unsigned short fragment);
 
 #endif
