@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include "device.h"
@@ -48,6 +49,7 @@
 #define VOLUME (7.0/8)
 
 #define SQ(x) ((x)*(x))
+#define TARGET_UNKNOWN INFINITY
 
 /*
  * Return: the cubic interpolation of the sample at position 2 + mu
@@ -66,6 +68,24 @@ static inline double cubic_interpolate(double y[4], double mu)
     return (a0 * mu * mu2) + (a1 * mu2) + (a2 * mu) + a3;
 }
 
+/*
+ * Return: Random dither, between -0.5 and 0.5
+ */
+
+double dither(void)
+{
+    short bit;
+    static short x = 0xbabe;
+
+    /* Use a 16-bit maximal-length LFSR as our random number.
+     * This is faster than rand() */
+
+    bit = (x ^ (x >> 2) ^ (x >> 3) ^ (x >> 5)) & 1;
+    x = x >> 1 | (bit << 15);
+
+    return (double)x / 65536 - 0.5; /* not quite whole range */
+}
+
 /*
  * Build a block of PCM audio, resampled from the track
  *
@@ -77,7 +97,7 @@ static inline double cubic_interpolate(double y[4], double mu)
  */
 
 static double build_pcm(signed short *pcm, unsigned samples, double sample_dt,
-                        struct track_t *tr, double position, double pitch,
+                        struct track *tr, double position, double pitch,
                         double start_vol, double end_vol)
 {
     int s;
@@ -118,8 +138,7 @@ static double build_pcm(signed short *pcm, unsigned samples, double sample_dt,
         for (c = 0; c < PLAYER_CHANNELS; c++) {
             double v;
 
-            v = vol * cubic_interpolate(i[c], f)
-                + (double)(rand() % 32768) / 32768 - 0.5; /* dither */
+            v = vol * cubic_interpolate(i[c], f) + dither();
 
             if (v > SHRT_MAX) {
                 *pcm++ = SHRT_MAX;
@@ -137,11 +156,26 @@ static double build_pcm(signed short *pcm, unsigned samples, double sample_dt,
     return sample_dt * pitch * samples;
 }
 
+/*
+ * Equivalent to build_pcm, but for use when the track is
+ * not available
+ *
+ * Return: number of seconds advanced in the audio track
+ * Post: buffer at pcm is filled with silence
+ */
+
+static double build_silence(signed short *pcm, unsigned samples,
+                            double sample_dt, double pitch)
+{
+    memset(pcm, '\0', sizeof(*pcm) * PLAYER_CHANNELS * samples);
+    return sample_dt * pitch * samples;
+}
+
 /*
  * Change the timecoder used by this playback
  */
 
-void player_set_timecoder(struct player_t *pl, struct timecoder_t *tc)
+void player_set_timecoder(struct player *pl, struct timecoder *tc)
 {
     assert(tc != NULL);
     pl->timecoder = tc;
@@ -153,18 +187,21 @@ void player_set_timecoder(struct player_t *pl, struct timecoder_t *tc)
  * Post: player is initialised
  */
 
-void player_init(struct player_t *pl, unsigned int sample_rate,
-                 struct track_t *track, struct timecoder_t *tc)
+void player_init(struct player *pl, unsigned int sample_rate,
+                 struct track *track, struct timecoder *tc)
 {
     assert(track != NULL);
     assert(sample_rate != 0);
+
+    spin_init(&pl->lock);
+
     pl->sample_dt = 1.0 / sample_rate;
     pl->track = track;
     player_set_timecoder(pl, tc);
 
     pl->position = 0.0;
     pl->offset = 0.0;
-    pl->target_valid = false;
+    pl->target_position = TARGET_UNKNOWN;
     pl->last_difference = 0.0;
 
     pl->pitch = 0.0;
@@ -177,15 +214,17 @@ void player_init(struct player_t *pl, unsigned int sample_rate,
  * Post: no resources are allocated by the player
  */
 
-void player_clear(struct player_t *pl)
+void player_clear(struct player *pl)
 {
+    spin_clear(&pl->lock);
+    track_put(pl->track);
 }
 
 /*
  * Enable or disable timecode control
  */
 
-void player_set_timecode_control(struct player_t *pl, bool on)
+void player_set_timecode_control(struct player *pl, bool on)
 {
     if (on && !pl->timecode_control)
         pl->recalibrate = true;
@@ -198,7 +237,7 @@ void player_set_timecode_control(struct player_t *pl, bool on)
  * Return: the new state of timecode control
  */
 
-bool player_toggle_timecode_control(struct player_t *pl)
+bool player_toggle_timecode_control(struct player *pl)
 {
     pl->timecode_control = !pl->timecode_control;
     if (pl->timecode_control)
@@ -206,13 +245,89 @@ bool player_toggle_timecode_control(struct player_t *pl)
     return pl->timecode_control;
 }
 
+double player_get_position(struct player *pl)
+{
+    return pl->position;
+}
+
+double player_get_elapsed(struct player *pl)
+{
+    return pl->position - pl->offset;
+}
+
+double player_get_remain(struct player *pl)
+{
+    return (double)pl->track->length / pl->track->rate
+        + pl->offset - pl->position;
+}
+
+bool player_is_active(const struct player *pl)
+{
+    return (fabs(pl->pitch) > 0.01);
+}
+
+/*
+ * Cue to the zero position of the track
+ */
+
+void player_recue(struct player *pl)
+{
+    pl->offset = pl->position;
+}
+
+/*
+ * Set the track used for the playback
+ *
+ * Pre: caller holds reference on track
+ * Post: caller does not hold reference on track
+ */
+
+void player_set_track(struct player *pl, struct track *track)
+{
+    struct track *x;
+
+    assert(track != NULL);
+    assert(track->refcount > 0);
+
+    spin_lock(&pl->lock); /* Synchronise with the playback thread */
+    x = pl->track;
+    pl->track = track;
+    spin_unlock(&pl->lock);
+
+    track_put(x); /* discard the old track */
+}
+
+/*
+ * Set the playback of one player to match another, used
+ * for "instant doubles" and beat juggling
+ */
+
+void player_clone(struct player *pl, const struct player *from)
+{
+    double elapsed;
+    struct track *x, *t;
+
+    elapsed = from->position - from->offset;
+    pl->offset = pl->position - elapsed;
+
+    t = from->track;
+    track_get(t);
+
+    spin_lock(&pl->lock);
+    x = pl->track;
+    pl->track = t;
+    spin_unlock(&pl->lock);
+
+    track_put(x);
+}
+
 /*
  * Synchronise to the position and speed given by the timecoder
  *
  * Return: 0 on success or -1 if the timecoder is not currently valid
  */
 
-static int sync_to_timecode(struct player_t *pl)
+static int sync_to_timecode(struct player *pl)
 {
     double when, tcpos;
     signed int timecode;
@@ -230,14 +345,12 @@ static int sync_to_timecode(struct player_t *pl)
     pl->pitch = timecoder_get_pitch(pl->timecoder);
 
     /* If we can read an absolute time from the timecode, then use it */
-    
-    if (timecode == -1)
-	pl->target_valid = false;
 
-    else {
+    if (timecode == -1) {
+	pl->target_position = TARGET_UNKNOWN;
+    } else {
         tcpos = (double)timecode / timecoder_get_resolution(pl->timecoder);
         pl->target_position = tcpos + pl->pitch * when;
-	pl->target_valid = true;
     }
 
     return 0;
@@ -248,20 +361,43 @@ static int sync_to_timecode(struct player_t *pl)
  * affecting the audio playback position
  */
 
-static void calibrate_to_timecode_position(struct player_t *pl)
+static void calibrate_to_timecode_position(struct player *pl)
 {
-    assert(pl->target_valid);
+    assert(pl->target_position != TARGET_UNKNOWN);
     pl->offset += pl->target_position - pl->position;
     pl->position = pl->target_position;
 }
 
-/*
- * Cue to the zero position of the track
- */
-
-void player_recue(struct player_t *pl)
+void retarget(struct player *pl)
 {
-    pl->offset = pl->position;
+    double diff;
+
+    if (pl->recalibrate) {
+        calibrate_to_timecode_position(pl);
+        pl->recalibrate = false;
+    }
+
+    /* Calculate the pitch compensation required to get us back on
+     * track with the absolute timecode position */
+
+    diff = pl->position - pl->target_position;
+    pl->last_difference = diff; /* to print in user interface */
+
+    if (fabs(diff) > SKIP_THRESHOLD) {
+
+        /* Jump the track to the time */
+
+        pl->position = pl->target_position;
+        fprintf(stderr, "Seek to new position %.2lfs.\n", pl->position);
+
+    } else if (fabs(pl->pitch) > SYNC_PITCH) {
+
+        /* Re-calculate the drift between the timecoder pitch from
+         * the sine wave and the timecode values */
+
+        pl->sync_pitch = pl->pitch / (diff / SYNC_TIME + pl->pitch);
+
+    }
 }
 
 /*
@@ -274,10 +410,9 @@ void player_recue(struct player_t *pl)
  * Post: buffer at pcm is filled with the given number of samples
  */
 
-void player_collect(struct player_t *pl, signed short *pcm, unsigned samples)
+void player_collect(struct player *pl, signed short *pcm, unsigned samples)
 {
-    double diff;
-    float dt, target_volume;
+    double r, pitch, dt, target_volume;
 
     dt = pl->sample_dt * samples;
 
@@ -286,44 +421,20 @@ void player_collect(struct player_t *pl, signed short *pcm, unsigned samples)
             pl->timecode_control = false;
     }
 
-    if (!pl->target_valid) {
+    if (pl->target_position != TARGET_UNKNOWN) {
 
-        /* Without timecode sync, tend sync_pitch towards 1.0, to
+        /* Bias the pitch towards a known target, and acknowledge that
+         * we did so */
+
+        retarget(pl);
+        pl->target_position = TARGET_UNKNOWN;
+
+    } else {
+
+        /* Without a known target, tend sync_pitch towards 1.0, to
          * avoid using outlier values from scratching for too long */
 
         pl->sync_pitch += dt / (SYNC_RC + dt) * (1.0 - pl->sync_pitch);
-
-    } else {
-        if (pl->recalibrate) {
-            calibrate_to_timecode_position(pl);
-            pl->recalibrate = false;
-        }
-
-        /* Calculate the pitch compensation required to get us back on
-         * track with the absolute timecode position */
-
-        diff = pl->position - pl->target_position;
-        pl->last_difference = diff; /* to print in user interface */
-        
-        if (fabs(diff) > SKIP_THRESHOLD) {
-
-            /* Jump the track to the time */
-            
-            pl->position = pl->target_position;
-            fprintf(stderr, "Seek to new position %.2lfs.\n", pl->position);
-
-        } else if (fabs(pl->pitch) > SYNC_PITCH) {
-
-            /* Re-calculate the drift between the timecoder pitch from
-             * the sine wave and the timecode values */
-
-            pl->sync_pitch = pl->pitch / (diff / SYNC_TIME + pl->pitch);
-
-        }
-
-        /* Acknowledge that we've accounted for the target position */
-        
-        pl->target_valid = false;
     }
 
     target_volume = fabs(pl->pitch) * VOLUME;
@@ -332,11 +443,17 @@ void player_collect(struct player_t *pl, signed short *pcm, unsigned samples)
 
     /* Sync pitch is applied post-filtering */
 
-    pl->position += build_pcm(pcm, samples, pl->sample_dt,
-			      pl->track,
-                              pl->position - pl->offset,
-                              pl->pitch * pl->sync_pitch,
-                              pl->volume, target_volume);
-    
+    pitch = pl->pitch * pl->sync_pitch;
+
+    if (!spin_try_lock(&pl->lock)) {
+        r = build_silence(pcm, samples, pl->sample_dt, pitch);
+    } else {
+        r = build_pcm(pcm, samples, pl->sample_dt, pl->track,
+                      pl->position - pl->offset, pitch,
+                      pl->volume, target_volume);
+        spin_unlock(&pl->lock);
+    }
+
+    pl->position += r;
     pl->volume = target_volume;
 }
