@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,46 @@
 
 #define REALTIME_PRIORITY 80
 
+static pthread_key_t key;
+
+/*
+ * Put in place checks for realtime and non-realtime threads
+ */
+
+int rt_global_init(void)
+{
+    int r;
+
+    r = pthread_key_create(&key, NULL);
+    if (r != 0) {
+        errno = r;
+        perror("pthread_key_create");
+        return -1;
+    }
+
+    if (pthread_setspecific(key, (void*)false) != 0)
+        abort();
+
+    return 0;
+}
+
+/*
+ * Check for programmer error
+ *
+ * Pre: the current thread is non realtime
+ */
+
+void rt_not_allowed()
+{
+    bool rt;
+
+    rt = (bool)pthread_getspecific(key);
+    if (rt) {
+        fprintf(stderr, "Realtime thread called a blocking function\n");
+        abort();
+    }
+}
+
 /*
  * Raise the priority of the current thread
  *
@@ -56,6 +96,12 @@ static int raise_priority()
         return -1;
     }
 
+    /* Set thread local storage to show that this thread is
+     * realtime, for assertions later */
+
+    if (pthread_setspecific(key, (void*)true) != 0)
+        abort();
+
     return 0;
 }
 
@@ -63,7 +109,7 @@ static int raise_priority()
  * The realtime thread
  */
 
-static void rt_main(struct rt_t *rt)
+static void rt_main(struct rt *rt)
 {
     int r;
     size_t n;
@@ -100,7 +146,7 @@ static void* launch(void *p)
  * Initialise state of realtime handler
  */
 
-void rt_init(struct rt_t *rt)
+void rt_init(struct rt *rt)
 {
     rt->finished = false;
     rt->ndv = 0;
@@ -111,7 +157,7 @@ void rt_init(struct rt_t *rt)
  * Clear resources associated with the realtime handler
  */
 
-void rt_clear(struct rt_t *rt)
+void rt_clear(struct rt *rt)
 {
 }
 
@@ -122,7 +168,7 @@ void rt_clear(struct rt_t *rt)
  * Post: if 0 is returned the device is added
  */
 
-int rt_add_device(struct rt_t *rt, struct device *dv)
+int rt_add_device(struct rt *rt, struct device *dv)
 {
     ssize_t z;
 
@@ -157,7 +203,7 @@ int rt_add_device(struct rt_t *rt, struct device *dv)
  * Return: -1 on error, otherwise 0
  */
 
-int rt_start(struct rt_t *rt)
+int rt_start(struct rt *rt)
 {
     size_t n;
 
@@ -165,6 +211,8 @@ int rt_start(struct rt_t *rt)
      * poll() then launch the realtime thread to handle them */
 
     if (rt->npt > 0) {
+        int r;
+
         fprintf(stderr, "Launching realtime thread to handle devices...\n");
 
         if (sem_init(&rt->sem, 0, 0) == -1) {
@@ -172,7 +220,9 @@ int rt_start(struct rt_t *rt)
             return -1;
         }
 
-        if (pthread_create(&rt->ph, NULL, launch, (void*)rt)) {
+        r = pthread_create(&rt->ph, NULL, launch, (void*)rt);
+        if (r != 0) {
+            errno = r;
             perror("pthread_create");
             if (sem_destroy(&rt->sem) == -1)
                 abort();
@@ -203,7 +253,7 @@ int rt_start(struct rt_t *rt)
  * Stop realtime handling, which was previously started by rt_start()
  */
 
-void rt_stop(struct rt_t *rt)
+void rt_stop(struct rt *rt)
 {
     size_t n;
 
