@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +30,7 @@
  * rt_stop
  */
 
-struct rt_t {
+struct rt {
     pthread_t ph;
     sem_t sem;
     bool finished;
@@ -42,12 +42,15 @@ struct rt_t {
     struct pollfd pt[32];
 };
 
-void rt_init(struct rt_t *rt);
-void rt_clear(struct rt_t *rt);
+int rt_global_init();
+void rt_not_allowed();
 
-int rt_add_device(struct rt_t *rt, struct device *dv);
+void rt_init(struct rt *rt);
+void rt_clear(struct rt *rt);
 
-int rt_start(struct rt_t *rt);
-void rt_stop(struct rt_t *rt);
+int rt_add_device(struct rt *rt, struct device *dv);
+
+int rt_start(struct rt *rt);
+void rt_stop(struct rt *rt);
 
 #endif
