@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,65 +20,59 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/poll.h>
 
+#include "list.h"
+#include "mutex.h"
+#include "realtime.h"
 #include "rig.h"
-#include "track.h"
 
 #define EVENT_WAKE 0
 #define EVENT_QUIT 1
 
-int rig_init(struct rig_t *rig)
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof(*x))
+
+static int event[2]; /* pipe to wake up service thread */
+static struct list tracks = LIST_INIT(tracks);
+mutex lock;
+
+int rig_init()
 {
     /* Create a pipe which will be used to wake us from other threads */
 
-    if (pipe(rig->event) == -1) {
+    if (pipe(event) == -1) {
         perror("pipe");
         return -1;
     }
 
-    if (fcntl(rig->event[0], F_SETFL, O_NONBLOCK) == -1) {
+    if (fcntl(event[0], F_SETFL, O_NONBLOCK) == -1) {
         perror("fcntl");
-        if (close(rig->event[1]) == -1)
+        if (close(event[1]) == -1)
             abort();
-        if (close(rig->event[0]) == -1)
+        if (close(event[0]) == -1)
             abort();
         return -1;
     }
 
-    rig->ntrack = 0;
+    mutex_init(&lock);
 
     return 0;
 }
 
-void rig_clear(struct rig_t *rig)
+void rig_clear()
 {
-    if (close(rig->event[0]) == -1)
+    mutex_clear(&lock);
+
+    if (close(event[0]) == -1)
         abort();
-    if (close(rig->event[1]) == -1)
+    if (close(event[1]) == -1)
         abort();
 }
 
-/*
- * Add a track to be monitored
- *
- * This function is not thread or signal safe to rig_main() as
- * is only used as part of initialisation.
- *
- * Pre: less than MAX_TRACKS tracks already added to this rig
- */
-
-void rig_add_track(struct rig_t *rig, struct track_t *track)
-{
-    assert(rig->ntrack < sizeof rig->track);
-
-    rig->track[rig->ntrack] = track;
-    rig->ntrack++;
-}
-
 /*
  * Main thread which handles input and output
  *
@@ -92,34 +86,44 @@ void rig_add_track(struct rig_t *rig, struct track_t *track)
  * I/O), the rig will also be responsible for them.
  */
 
-int rig_main(struct rig_t *rig)
+int rig_main()
 {
-    int r;
-    size_t n;
-    struct pollfd pt[4], *pe;
+    struct pollfd pt[4];
 
-    assert(sizeof pt >= rig->ntrack + 1);
+    /* Monitor event pipe from external threads */
+
+    pt[0].fd = event[0];
+    pt[0].revents = 0;
+    pt[0].events = POLLIN;
+
+    mutex_lock(&lock);
 
     for (;;) { /* exit via EVENT_QUIT */
-        pe = pt;
+        int r;
+        struct pollfd *pe;
+        struct track *track;
 
-        /* ppoll() is not widely available, so use a pipe between this
-         * thread and the outside. A single byte wakes up poll() to
-         * inform us that new file descriptors need to be polled */
-
-        pe->fd = rig->event[0];
-        pe->revents = 0;
-        pe->events = POLLIN;
-        pe++;
+        pe = &pt[1];
 
         /* Fetch file descriptors to monitor from each track */
 
-        for (n = 0; n < rig->ntrack; n++)
-            pe += track_pollfd(rig->track[n], pe);
+        list_for_each(track, &tracks, rig) {
+            track_pollfd(track, pe);
+            pe++;
+
+            /* If we ran out of file descriptors, ignore the
+             * oldest until we have space in the poll table */
+
+            if (pe >= pt + ARRAY_SIZE(pt))
+                break;
+        }
+
+        mutex_unlock(&lock);
 
         r = poll(pt, pe - pt, -1);
         if (r == -1) {
             if (errno == EINTR) {
+                mutex_lock(&lock);
                 continue;
             } else {
                 perror("poll");
@@ -131,10 +135,10 @@ int rig_main(struct rig_t *rig)
 
         if (pt[0].revents != 0) {
             for (;;) {
-                char event;
+                char e;
                 size_t z;
 
-                z = read(rig->event[0], &event, 1);
+                z = read(event[0], &e, 1);
                 if (z == -1) {
                     if (errno == EAGAIN) {
                         break;
@@ -144,7 +148,7 @@ int rig_main(struct rig_t *rig)
                     }
                 }
 
-                switch (event) {
+                switch (e) {
                 case EVENT_WAKE:
                     break;
 
@@ -157,10 +161,16 @@ int rig_main(struct rig_t *rig)
             }
         }
 
+        mutex_lock(&lock);
+
         /* Do any reading and writing on all tracks */
 
-        for (n = 0; n < rig->ntrack; n++)
-            track_handle(rig->track[n]);
+        list_for_each(track, &tracks, rig) {
+            if (track_handle(track)) {
+                list_del(&track->rig);
+                track_put(track);
+            }
+        }
     }
  finish:
 
@@ -171,29 +181,38 @@ int rig_main(struct rig_t *rig)
  * Post a simple event into the rig event loop
  */
 
-static int post_event(struct rig_t *rig, char event)
+static int post_event(char e)
 {
-    if (write(rig->event[1], &event, 1) == -1) {
+    rt_not_allowed();
+
+    if (write(event[1], &e, 1) == -1) {
         perror("write");
         return -1;
     }
+
     return 0;
 }
 
-/*
- * Wake up the rig to inform it that the poll table has changed
- */
-
-int rig_awaken(struct rig_t *rig)
-{
-    return post_event(rig, EVENT_WAKE);
-}
-
 /*
  * Ask the rig to exit from another thread or signal handler
  */
 
-int rig_quit(struct rig_t *rig)
+int rig_quit()
 {
-    return post_event(rig, EVENT_QUIT);
+    return post_event(EVENT_QUIT);
+}
+
+/*
+ * Add a track to be handled until import has completed
+ */
+
+void rig_post_track(struct track *t)
+{
+    track_get(t);
+
+    mutex_lock(&lock);
+    list_add(&t->rig, &tracks);
+    mutex_unlock(&lock);
+
+    post_event(EVENT_WAKE);
 }
