@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>,
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>,
  *                    Yves Adler <yves.adler@googlemail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #include "selector.h"
 
 
-static void scroll_reset(struct scroll_t *s)
+static void scroll_reset(struct scroll *s)
 {
     s->lines = 0;
     s->offset = 0;
@@ -36,7 +36,7 @@ static void scroll_reset(struct scroll_t *s)
 /* Set the number of lines displayed on screen. The current selection
  * is moved to within range. */
 
-static void scroll_set_lines(struct scroll_t *s, unsigned int lines)
+static void scroll_set_lines(struct scroll *s, unsigned int lines)
 {
     s->lines = lines;
     if (s->selected >= s->offset + s->lines)
@@ -52,7 +52,7 @@ static void scroll_set_lines(struct scroll_t *s, unsigned int lines)
 /* Set the number of entries in the list which backs the scrolling
  * display. Bring the current selection within the bounds given. */
 
-static void scroll_set_entries(struct scroll_t *s, unsigned int entries)
+static void scroll_set_entries(struct scroll *s, unsigned int entries)
 {
     s->entries = entries;
     if (s->selected >= s->entries)
@@ -73,7 +73,7 @@ static void scroll_set_entries(struct scroll_t *s, unsigned int entries)
 /* Scroll the selection up by n lines. Move the window offset if
  * needed */
 
-static void scroll_up(struct scroll_t *s, unsigned int n)
+static void scroll_up(struct scroll *s, unsigned int n)
 {
     s->selected -= n;
     if (s->selected < 0)
@@ -89,7 +89,7 @@ static void scroll_up(struct scroll_t *s, unsigned int n)
 }
 
 
-static void scroll_down(struct scroll_t *s, unsigned int n)
+static void scroll_down(struct scroll *s, unsigned int n)
 {
     s->selected += n;
     if (s->selected >= s->entries)
@@ -107,7 +107,7 @@ static void scroll_down(struct scroll_t *s, unsigned int n)
 
 /* Scroll to the first entry on the list */
 
-static void scroll_first(struct scroll_t *s)
+static void scroll_first(struct scroll *s)
 {
     s->selected = 0;
     s->offset = 0;
@@ -116,7 +116,7 @@ static void scroll_first(struct scroll_t *s)
 
 /* Scroll to the final entry on the list */
 
-static void scroll_last(struct scroll_t *s)
+static void scroll_last(struct scroll *s)
 {
     s->selected = s->entries - 1;
     s->offset = s->selected - s->lines + 1;
@@ -127,32 +127,28 @@ static void scroll_last(struct scroll_t *s)
 
 /* Scroll to an entry by index */
 
-static void scroll_to(struct scroll_t *s, unsigned int n)
+static void scroll_to(struct scroll *s, unsigned int n)
 {
+    int p;
+
+    assert(s->selected != -1);
+    assert(n < s->entries);
+
+    /* Retain the on-screen position of the current selection */
+
+    p = s->selected - s->offset;
     s->selected = n;
+    s->offset = s->selected - p;
 
-    /* Move the viewing offset down, if necessary */
-
-    if (s->selected >= s->offset + s->lines) {
-        s->offset = s->selected - s->lines / 2;
-        if (s->offset + s->lines > s->entries)
-            s->offset = s->entries - s->lines;
-    }
-
-    /* Move the viewing offset up, if necessary */
-
-    if (s->selected < s->offset) {
-        s->offset = s->selected - s->lines / 2 + 1;
-        if (s->offset < 0)
-            s->offset = 0;
-    }
+    if (s->offset < 0)
+        s->offset = 0;
 }
 
 
 /* Return the index of the current selected list entry, or -1 if
  * no current selection */
 
-static int scroll_current(struct scroll_t *s)
+static int scroll_current(struct scroll *s)
 {
     if (s->entries == 0) {
         return -1;
@@ -162,16 +158,35 @@ static int scroll_current(struct scroll_t *s)
 }
 
 
+/* Scroll to an entry if it can be found, otherwise leave our position
+ * unchanged */
+
+static void retain_position(struct selector *sel, struct record *x)
+{
+    size_t n;
+    struct listing *l;
+
+    l = sel->view_listing;
+
+    for (n = 0; n < l->size; n++) {
+        if (l->record[n] == x) {
+            scroll_to(&sel->records, n);
+            return;
+        }
+    }
+}
+
+
 /* Return the listing which acts as the starting point before
  * string matching, based on the current crate */
 
-static struct listing_t* initial(struct selector_t *sel)
+static struct listing* initial(struct selector *sel)
 {
     return &sel->library->crate[sel->crates.selected]->listing;
 }
 
 
-void selector_init(struct selector_t *sel, struct library_t *lib)
+void selector_init(struct selector *sel, struct library *lib)
 {
     sel->library = lib;
 
@@ -194,57 +209,57 @@ void selector_init(struct selector_t *sel, struct library_t *lib)
 }
 
 
-void selector_clear(struct selector_t *sel)
+void selector_clear(struct selector *sel)
 {
     listing_clear(&sel->listing_a);
     listing_clear(&sel->listing_b);
 }
 
 
-void selector_set_lines(struct selector_t *sel, unsigned int lines)
+void selector_set_lines(struct selector *sel, unsigned int lines)
 {
     scroll_set_lines(&sel->crates, lines);
     scroll_set_lines(&sel->records, lines);
 }
 
 
-void selector_up(struct selector_t *sel)
+void selector_up(struct selector *sel)
 {
     scroll_up(&sel->records, 1);
 }
 
 
-void selector_down(struct selector_t *sel)
+void selector_down(struct selector *sel)
 {
     scroll_down(&sel->records, 1);
 }
 
 
-void selector_page_up(struct selector_t *sel)
+void selector_page_up(struct selector *sel)
 {
     scroll_up(&sel->records, sel->records.lines);
 }
 
 
-void selector_page_down(struct selector_t *sel)
+void selector_page_down(struct selector *sel)
 {
     scroll_down(&sel->records, sel->records.lines);
 }
 
 
-void selector_top(struct selector_t *sel)
+void selector_top(struct selector *sel)
 {
     scroll_first(&sel->records);
 }
 
 
-void selector_bottom(struct selector_t *sel)
+void selector_bottom(struct selector *sel)
 {
     scroll_last(&sel->records);
 }
 
 
-struct record_t* selector_current(struct selector_t *sel)
+struct record* selector_current(struct selector *sel)
 {
     int i;
 
@@ -260,14 +275,14 @@ struct record_t* selector_current(struct selector_t *sel)
 /* When the crate has changed, update the current listing to reflect
  * the crate and the search criteria */
 
-static void crate_has_changed(struct selector_t *sel)
+static void crate_has_changed(struct selector *sel)
 {
     (void)listing_match(initial(sel), sel->view_listing, sel->search);
     scroll_set_entries(&sel->records, sel->view_listing->entries);
 }
 
 
-void selector_prev(struct selector_t *sel)
+void selector_prev(struct selector *sel)
 {
     scroll_up(&sel->crates, 1);
     sel->toggled = false;
@@ -275,7 +290,7 @@ void selector_prev(struct selector_t *sel)
 }
 
 
-void selector_next(struct selector_t *sel)
+void selector_next(struct selector *sel)
 {
     scroll_down(&sel->crates, 1);
     sel->toggled = false;
@@ -285,7 +300,7 @@ void selector_next(struct selector_t *sel)
 
 /* Toggle between the current crate and the 'all' crate */
 
-void selector_toggle(struct selector_t *sel)
+void selector_toggle(struct selector *sel)
 {
     if (!sel->toggled) {
         sel->toggle_back = scroll_current(&sel->crates);
@@ -302,24 +317,31 @@ void selector_toggle(struct selector_t *sel)
 /* Expand the search. Do not disrupt the running process on memory
  * allocation failure, leave the view listing incomplete */
 
-void selector_search_expand(struct selector_t *sel)
+void selector_search_expand(struct selector *sel)
 {
+    struct record *c;
+
     if (sel->search_len == 0)
         return;
 
     sel->search[--sel->search_len] = '\0';
 
+    c = selector_current(sel);
+
     (void)listing_match(initial(sel), sel->view_listing, sel->search);
     scroll_set_entries(&sel->records, sel->view_listing->entries);
+
+    if (c != NULL)
+        retain_position(sel, c);
 }
 
 
 /* Refine the search. Do not distrupt the running process on memory
  * allocation failure, leave the view listing incomplete */
 
-void selector_search_refine(struct selector_t *sel, char key)
+void selector_search_refine(struct selector *sel, char key)
 {
-    struct listing_t *tmp;
+    struct listing *tmp;
 
     if (sel->search_len >= sizeof(sel->search) - 1) /* would overflow */
         return;
