@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>,
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>,
  *                    Yves Adler <yves.adler@googlemail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -30,18 +30,18 @@
 /* Managed context of a scrolling window, of a number of fixed-height
  * lines, backed by a list of a known number of entries */
 
-struct scroll_t {
+struct scroll {
     int lines, offset, entries, selected;
 };
 
-struct selector_t {
-    struct library_t *library;
-    struct listing_t
+struct selector {
+    struct library *library;
+    struct listing
         *view_listing, /* base_listing + search filter applied */
         *swap_listing, /* used to swap between a and b listings */
         listing_a, listing_b;
 
-    struct scroll_t records, crates;
+    struct scroll records, crates;
     bool toggled;
     int toggle_back;
 
@@ -49,25 +49,25 @@ struct selector_t {
     char search[256];
 };
 
-void selector_init(struct selector_t *sel, struct library_t *lib);
-void selector_clear(struct selector_t *sel);
+void selector_init(struct selector *sel, struct library *lib);
+void selector_clear(struct selector *sel);
 
-void selector_set_lines(struct selector_t *sel, unsigned int lines);
+void selector_set_lines(struct selector *sel, unsigned int lines);
 
-void selector_up(struct selector_t *sel);
-void selector_down(struct selector_t *sel);
-void selector_page_up(struct selector_t *sel);
-void selector_page_down(struct selector_t *sel);
-void selector_top(struct selector_t *sel);
-void selector_bottom(struct selector_t *sel);
+void selector_up(struct selector *sel);
+void selector_down(struct selector *sel);
+void selector_page_up(struct selector *sel);
+void selector_page_down(struct selector *sel);
+void selector_top(struct selector *sel);
+void selector_bottom(struct selector *sel);
 
-struct record_t* selector_current(struct selector_t *sel);
+struct record* selector_current(struct selector *sel);
 
-void selector_prev(struct selector_t *sel);
-void selector_next(struct selector_t *sel);
-void selector_toggle(struct selector_t *sel);
+void selector_prev(struct selector *sel);
+void selector_next(struct selector *sel);
+void selector_toggle(struct selector *sel);
 
-void selector_search_expand(struct selector_t *sel);
-void selector_search_refine(struct selector_t *sel, char key);
+void selector_search_expand(struct selector *sel);
+void selector_search_refine(struct selector *sel, char key);
 
 #endif
