@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "debug.h"
 #include "timecoder.h"
 
 #define ZERO_THRESHOLD 128
@@ -48,7 +49,7 @@
 #define SWITCH_PRIMARY 0x2 /* use left channel (not right) as primary */
 #define SWITCH_POLARITY 0x4 /* read bit values in negative (not positive) */
 
-static struct timecode_def_t timecodes[] = {
+static struct timecode_def timecodes[] = {
     {
         .name = "serato_2a",
         .desc = "Serato 2nd Ed., side A",
@@ -159,7 +160,7 @@ static inline bits_t lfsr(bits_t code, bits_t taps)
  * are generated at the least-significant bit.
  */
 
-static inline bits_t fwd(bits_t current, struct timecode_def_t *def)
+static inline bits_t fwd(bits_t current, struct timecode_def *def)
 {
     bits_t l;
 
@@ -173,7 +174,7 @@ static inline bits_t fwd(bits_t current, struct timecode_def_t *def)
  * Linear Feedback Shift Register in the reverse direction
  */
 
-static inline bits_t rev(bits_t current, struct timecode_def_t *def)
+static inline bits_t rev(bits_t current, struct timecode_def *def)
 {
     bits_t l, mask;
 
@@ -190,7 +191,7 @@ static inline bits_t rev(bits_t current, struct timecode_def_t *def)
  * Return: -1 if not enough memory could be allocated, otherwise 0
  */
 
-static int build_lookup(struct timecode_def_t *def)
+static int build_lookup(struct timecode_def *def)
 {
     unsigned int n;
     bits_t current, last;
@@ -208,11 +209,11 @@ static int build_lookup(struct timecode_def_t *def)
 
     for (n = 0; n < def->length; n++) {
         /* timecode must not wrap */
-        assert(lut_lookup(&def->lut, current) == (unsigned)-1);
+        dassert(lut_lookup(&def->lut, current) == (unsigned)-1);
         lut_push(&def->lut, current);
         last = current;
         current = fwd(current, def);
-        assert(rev(current, def) == last);
+        dassert(rev(current, def) == last);
     }
 
     def->lookup = true;
@@ -226,9 +227,9 @@ static int build_lookup(struct timecode_def_t *def)
  * Return: pointer to timecode definition, or NULL if not found
  */
 
-struct timecode_def_t* timecoder_find_definition(const char *name)
+struct timecode_def* timecoder_find_definition(const char *name)
 {
-    struct timecode_def_t *def, *end;
+    struct timecode_def *def, *end;
 
     def = &timecodes[0];
     end = def + ARRAY_SIZE(timecodes);
@@ -254,7 +255,7 @@ struct timecode_def_t* timecoder_find_definition(const char *name)
  */
 
 void timecoder_free_lookup(void) {
-    struct timecode_def_t *def, *end;
+    struct timecode_def *def, *end;
 
     def = &timecodes[0];
     end = def + ARRAY_SIZE(timecodes);
@@ -270,9 +271,9 @@ void timecoder_free_lookup(void) {
  * Initialise filter values for one channel
  */
 
-static void init_channel(struct timecoder_channel_t *ch)
+static void init_channel(struct timecoder_channel *ch)
 {
-    ch->positive = 0;
+    ch->positive = false;
     ch->zero = 0;
 }
 
@@ -282,7 +283,7 @@ static void init_channel(struct timecoder_channel_t *ch)
  * Return: -1 if the timecoder could not be initialised, otherwise 0
  */
 
-void timecoder_init(struct timecoder_t *tc, struct timecode_def_t *def,
+void timecoder_init(struct timecoder *tc, struct timecode_def *def,
                     double speed, unsigned int sample_rate)
 {
     assert(def != NULL);
@@ -315,7 +316,7 @@ void timecoder_init(struct timecoder_t *tc, struct timecode_def_t *def,
  * Clear resources associated with a timecode decoder
  */
 
-void timecoder_clear(struct timecoder_t *tc)
+void timecoder_clear(struct timecoder *tc)
 {
     assert(tc->mon == NULL);
 }
@@ -329,7 +330,7 @@ void timecoder_clear(struct timecoder_t *tc)
  * Return: -1 if not enough memory could be allocated, otherwise 0
  */
 
-int timecoder_monitor_init(struct timecoder_t *tc, int size)
+int timecoder_monitor_init(struct timecoder *tc, int size)
 {
     assert(tc->mon == NULL);
     tc->mon_size = size;
@@ -347,7 +348,7 @@ int timecoder_monitor_init(struct timecoder_t *tc, int size)
  * Clear the monitor on the given timecoder
  */
 
-void timecoder_monitor_clear(struct timecoder_t *tc)
+void timecoder_monitor_clear(struct timecoder *tc)
 {
     assert(tc->mon != NULL);
     free(tc->mon);
@@ -358,19 +359,19 @@ void timecoder_monitor_clear(struct timecoder_t *tc)
  * Update channel information with axis-crossings
  */
 
-static void detect_zero_crossing(struct timecoder_channel_t *ch,
+static void detect_zero_crossing(struct timecoder_channel *ch,
                                  signed int v, double alpha)
 {
     ch->crossing_ticker++;
 
-    ch->swapped = 0;
+    ch->swapped = false;
     if (v > ch->zero + ZERO_THRESHOLD && !ch->positive) {
-        ch->swapped = 1;
-        ch->positive = 1;
+        ch->swapped = true;
+        ch->positive = true;
         ch->crossing_ticker = 0;
     } else if (v < ch->zero - ZERO_THRESHOLD && ch->positive) {
-        ch->swapped = 1;
-        ch->positive = 0;
+        ch->swapped = true;
+        ch->positive = false;
         ch->crossing_ticker = 0;
     }
 
@@ -381,7 +382,7 @@ static void detect_zero_crossing(struct timecoder_channel_t *ch,
  * Plot the given sample value in the x-y monitor
  */
 
-static void update_monitor(struct timecoder_t *tc, signed int x, signed int y)
+static void update_monitor(struct timecoder *tc, signed int x, signed int y)
 {
     int px, py, p;
     double v, w;
@@ -414,7 +415,7 @@ static void update_monitor(struct timecoder_t *tc, signed int x, signed int y)
  * Extract the bitstream from the sample value
  */
 
-static void process_bitstream(struct timecoder_t *tc, signed int m)
+static void process_bitstream(struct timecoder *tc, signed int m)
 {
     bits_t b;
 
@@ -454,22 +455,18 @@ static void process_bitstream(struct timecoder_t *tc, signed int m)
 
     tc->ref_level = (tc->ref_level * (REF_PEAKS_AVG - 1) + m) / REF_PEAKS_AVG;
 
-#ifdef DEBUG_BITSTREAM
-    fprintf(stderr, "%+6d zero, %+6d (ref %+6d)\t= %d%c (%5d)\n",
-	    tc->primary.zero,
-	    m,
-	    tc->ref_level,
-	    b,
-	    tc->valid_counter == 0 ? 'x' : ' ',
-	    tc->valid_counter);
-#endif
+    debug("%+6d zero, %+6d (ref %+6d)\t= %d%c (%5d)\n",
+          tc->primary.zero,
+          m, tc->ref_level,
+	  b, tc->valid_counter == 0 ? 'x' : ' ',
+	  tc->valid_counter);
 }
 
 /*
  * Process a single sample from the incoming audio
  */
 
-static void process_sample(struct timecoder_t *tc,
+static void process_sample(struct timecoder *tc,
 			   signed int primary, signed int secondary)
 {
     signed int m; /* pcm sample, sum of two shorts */
@@ -532,7 +529,7 @@ static void process_sample(struct timecoder_t *tc,
  * Return: pointer to timecode definition
  */
 
-static struct timecode_def_t* next_definition(struct timecode_def_t *def)
+static struct timecode_def* next_definition(struct timecode_def *def)
 {
     assert(def != NULL);
 
@@ -551,7 +548,7 @@ static struct timecode_def_t* next_definition(struct timecode_def_t *def)
  * Change the timecode definition to the next available
  */
 
-void timecoder_cycle_definition(struct timecoder_t *tc)
+void timecoder_cycle_definition(struct timecoder *tc)
 {
     tc->def = next_definition(tc->def);
     tc->valid_counter = 0;
@@ -562,7 +559,7 @@ void timecoder_cycle_definition(struct timecoder_t *tc)
  * Submit and decode a block of PCM audio data to the timecode decoder
  */
 
-void timecoder_submit(struct timecoder_t *tc, signed short *pcm, size_t npcm)
+void timecoder_submit(struct timecoder *tc, signed short *pcm, size_t npcm)
 {
     while (npcm--) {
 	signed int primary, secondary;
@@ -594,7 +591,7 @@ void timecoder_submit(struct timecoder_t *tc, signed short *pcm, size_t npcm)
  * Post: if when != -1, *when contains the elapsed time in seconds
  */
 
-signed int timecoder_get_position(struct timecoder_t *tc, double *when)
+signed int timecoder_get_position(struct timecoder *tc, double *when)
 {
     signed int r;
 
