@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,14 +22,32 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "debug.h"
+#include "list.h"
+#include "realtime.h"
 #include "rig.h"
 #include "track.h"
 
 #define SAMPLE (sizeof(signed short) * TRACK_CHANNELS) /* bytes per sample */
 #define TRACK_BLOCK_PCM_BYTES (TRACK_BLOCK_SAMPLES * SAMPLE)
 
-#define LOCK(tr) pthread_mutex_lock(&(tr)->mx)
-#define UNLOCK(tr) pthread_mutex_unlock(&(tr)->mx)
+struct list tracks = LIST_INIT(tracks);
+
+/*
+ * An empty track is used rarely, and is easier than
+ * continuous checks for NULL throughout the code
+ */
+
+static struct track empty = {
+    .refcount = 1,
+
+    .rate = TRACK_RATE,
+    .bytes = 0,
+    .length = 0,
+    .blocks = 0,
+
+    .importing = false
+};
 
 /*
  * Allocate more memory
@@ -37,16 +55,18 @@
  * Return: -1 if memory could not be allocated, otherwize 0
  */
 
-static int more_space(struct track_t *tr)
+static int more_space(struct track *tr)
 {
-    struct track_block_t *block;
+    struct track_block *block;
+
+    rt_not_allowed();
 
     if (tr->blocks >= TRACK_MAX_BLOCKS) {
         fprintf(stderr, "Maximum track length reached.\n");
         return -1;
     }
 
-    block = malloc(sizeof(struct track_block_t));
+    block = malloc(sizeof(struct track_block));
     if (block == NULL) {
         perror("malloc");
         return -1;
@@ -54,8 +74,8 @@ static int more_space(struct track_t *tr)
 
     tr->block[tr->blocks++] = block;
 
-    fprintf(stderr, "Allocated new track block (%d blocks, %zu bytes).\n",
-            tr->blocks, tr->blocks * TRACK_BLOCK_SAMPLES * SAMPLE);
+    debug("Allocated new track block (%d blocks, %zu bytes).\n",
+          tr->blocks, tr->blocks * TRACK_BLOCK_SAMPLES * SAMPLE);
 
     return 0;
 }
@@ -67,7 +87,7 @@ static int more_space(struct track_t *tr)
  * Post: len contains the length of the buffer, in bytes
  */
 
-void* track_access_pcm(struct track_t *tr, size_t *len)
+void* track_access_pcm(struct track *tr, size_t *len)
 {
     unsigned int block;
     size_t fill;
@@ -94,22 +114,21 @@ void* track_access_pcm(struct track_t *tr, size_t *len)
  * given by track_access_pcm()
  */
 
-static void commit_pcm_samples(struct track_t *tr, unsigned int samples)
+static void commit_pcm_samples(struct track *tr, unsigned int samples)
 {
-    unsigned int fill;
+    unsigned int fill, n;
     signed short *pcm;
-    struct track_block_t *block;
+    struct track_block *block;
 
     block = tr->block[tr->length / TRACK_BLOCK_SAMPLES];
     fill = tr->length % TRACK_BLOCK_SAMPLES;
     pcm = block->pcm + TRACK_CHANNELS * fill;
 
     assert(samples <= TRACK_BLOCK_SAMPLES - fill);
-    tr->length += samples;
 
     /* Meter the new audio */
 
-    while (samples > 0) {
+    for (n = samples; n > 0; n--) {
         unsigned short v;
         unsigned int w;
 
@@ -138,8 +157,9 @@ static void commit_pcm_samples(struct track_t *tr, unsigned int samples)
 
         fill++;
         pcm += TRACK_CHANNELS;
-        samples--;
     }
+
+    tr->length += samples; /* other threads; write assumed to be atomic */
 }
 
 /*
@@ -152,32 +172,45 @@ static void commit_pcm_samples(struct track_t *tr, unsigned int samples)
  * from track_access_pcm()
  */
 
-void track_commit(struct track_t *tr, size_t len)
+void track_commit(struct track *tr, size_t len)
 {
     tr->bytes += len;
     commit_pcm_samples(tr, tr->bytes / SAMPLE - tr->length);
 }
 
 /*
- * Initialise object which will hold PCM audio data
+ * Initialise object which will hold PCM audio data, and start
+ * importing the data
  *
  * Post: track is initialised
  */
 
-void track_init(struct track_t *tr, const char *importer)
+static int track_init(struct track *t, const char *importer,
+                       const char *path)
 {
-    tr->importer = importer;
-    tr->artist = NULL;
-    tr->title = NULL;
+    t->refcount = 0;
 
-    tr->blocks = 0;
-    tr->importing = false;
-    tr->rate = TRACK_RATE;
+    t->blocks = 0;
+    t->rate = TRACK_RATE;
 
-    track_empty(tr);
+    t->bytes = 0;
+    t->length = 0;
+    t->ppm = 0;
+    t->overview = 0;
 
-    if (pthread_mutex_init(&tr->mx, NULL) != 0)
-        abort();
+    t->importing = true;
+    t->has_poll = false;
+
+    t->importer = importer;
+    t->path = path;
+
+    if (import_start(&t->import, t, importer, path) == -1)
+        return -1;
+
+    list_add(&t->tracks, &tracks);
+    rig_post_track(t);
+
+    return 0;
 }
 
 /*
@@ -189,71 +222,106 @@ void track_init(struct track_t *tr, const char *importer)
  * Pre: track is initialised
  */
 
-void track_clear(struct track_t *tr)
+static void track_clear(struct track *tr)
 {
     int n;
 
-    /* Force a cleanup of whichever state we are in */
-
-    if (tr->importing) {
-        import_terminate(&tr->import);
-        import_stop(&tr->import);
-    }
+    assert(!tr->importing);
 
     for (n = 0; n < tr->blocks; n++)
         free(tr->block[n]);
 
-    if (pthread_mutex_destroy(&tr->mx) != 0)
-        abort();
+    list_del(&tr->tracks);
 }
 
 /*
- * Make the given track empty
+ * Get a pointer to a track object already in memory
  *
- * This function does not deallocate the buffer, which is a quick
- * way to handle concurrency issues -- the realtime thread accessing
- * the audio.
- *
- * Post: track is 0 seconds in length
+ * Return: pointer, or NULL if no such track exists
  */
 
-void track_empty(struct track_t *tr)
+static struct track* track_get_again(const char *importer, const char *path)
 {
-    tr->bytes = 0;
-    tr->length = 0;
-    tr->ppm = 0;
-    tr->overview = 0;
-}
+    struct track *t;
 
-/*
- * Import audio from the given path
- *
- * Cleans up any existing import operation, and so can be called
- * when the track is in any initialised state.
- *
- * Post: track is importing
- */
-
-int track_import(struct track_t *tr, const char *path)
-{
-    int r;
-
-    LOCK(tr);
-
-    /* Abort any running import process */
-
-    if (tr->importing) {
-        import_terminate(&tr->import);
-        import_stop(&tr->import);
+    list_for_each(t, &tracks, tracks) {
+        if (t->importer == importer && t->path == path) {
+            track_get(t);
+            return t;
+        }
     }
 
-    /* Start the new import process */
+    return NULL;
+}
 
-    r = import_start(&tr->import, tr, tr->importer, path);
+/*
+ * Get a pointer to a track object for the given importer and path
+ *
+ * Return: pointer, or NULL if not enough resources
+ */
 
-    UNLOCK(tr);
+struct track* track_get_by_import(const char *importer, const char *path)
+{
+    struct track *t;
 
-    return r;
+    t = track_get_again(importer, path);
+    if (t != NULL)
+        return t;
+
+    t = malloc(sizeof *t);
+    if (t == NULL) {
+        perror("malloc");
+        return NULL;
+    }
+
+    if (track_init(t, importer, path) == -1) {
+        free(t);
+        return NULL;
+    }
+
+    track_get(t);
+
+    return t;
+}
+
+/*
+ * Get a pointer to a static track containing no audio
+ *
+ * Return: pointer
+ */
+
+struct track* track_get_empty(void)
+{
+    empty.refcount++;
+    return &empty;
+}
+
+void track_get(struct track *t)
+{
+    t->refcount++;
+}
+
+/*
+ * Finish use of a track object
+ */
+
+void track_put(struct track *t)
+{
+    t->refcount--;
+
+    /* When importing, a reference is held. If it's the
+     * only one remaining terminate it to save resources */
+
+    if (t->refcount == 1 && t->importing) {
+        import_terminate(&t->import);
+        return;
+    }
+
+    if (t->refcount == 0 && t != &empty) {
+        assert(!t->importing);
+        track_clear(t);
+        free(t);
+    }
 }
 
 /*
@@ -263,43 +331,33 @@ int track_import(struct track_t *tr, const char *path)
  * Post: *pe contains 0 or 1 file descriptors
  */
 
-size_t track_pollfd(struct track_t *tr, struct pollfd *pe)
+void track_pollfd(struct track *t, struct pollfd *pe)
 {
-    int r;
+    assert(t->importing);
 
-    LOCK(tr);
-
-    if (tr->importing) {
-        import_pollfd(&tr->import, pe);
-        tr->has_poll = true;
-        r = 1;
-    } else {
-        tr->has_poll = false;
-        r = 0;
-    }
-
-    UNLOCK(tr);
-
-    return r;
+    import_pollfd(&t->import, pe);
+    t->has_poll = true;
 }
 
 /*
  * Handle any file descriptor activity on this track
+ *
+ * Return: true if import has completed, otherwise false
  */
 
-void track_handle(struct track_t *tr)
+bool track_handle(struct track *tr)
 {
-    LOCK(tr);
+    /* A track may be added while poll() was waiting,
+     * in which case it has no return data from poll */
 
-    /* Number of file descriptors watched will fluctuate between zero
-     * and non-zero, so we need to use has_poll */
+    if (!tr->has_poll)
+        return false;
 
-    if (tr->importing && tr->has_poll) {
-        if (import_handle(&tr->import) == -1) {
-            import_stop(&tr->import);
-            tr->importing = false;
-        }
+    if (import_handle(&tr->import) == -1) {
+        import_stop(&tr->import);
+        tr->importing = false;
+        return true;
     }
 
-    UNLOCK(tr);
+    return false;
 }
