@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -49,8 +49,10 @@
 #define DEFAULT_SCANNER EXECDIR "/xwax-scan"
 #define DEFAULT_TIMECODE "serato_2a"
 
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof(*x))
+
 char *banner = "xwax " VERSION \
-    " (C) Copyright 2011 Mark Hills <mark@pogo.org.uk>";
+    " (C) Copyright 2012 Mark Hills <mark@pogo.org.uk>";
 
 static void usage(FILE *fd)
 {
@@ -60,6 +62,8 @@ static void usage(FILE *fd)
       "  -t <name>      Timecode name\n"
       "  -33            Use timecode at 33.3RPM (default)\n"
       "  -45            Use timecode at 45RPM\n"
+      "  -c             Protect against certain operations while playing\n"
+      "  -u             Allow all operations when playing\n"
       "  -i <program>   Importer (default '%s')\n"
       "  -s <program>   Library scanner (default '%s')\n"
       "  -h             Display this message\n\n",
@@ -100,15 +104,14 @@ static void usage(FILE *fd)
 int main(int argc, char *argv[])
 {
     int r, n, decks;
-    char *importer, *scanner;
+    const char *importer, *scanner;
     double speed;
-    struct timecode_def_t *timecode;
+    struct timecode_def *timecode;
+    bool protect;
 
-    struct deck_t deck[3];
-    struct rig_t rig;
-    struct rt_t rt;
-    struct interface_t iface;
-    struct library_t library;
+    struct deck deck[3];
+    struct rt rt;
+    struct library library;
 
 #if defined WITH_OSS || WITH_ALSA
     char *endptr;
@@ -125,7 +128,10 @@ int main(int argc, char *argv[])
 
     fprintf(stderr, "%s\n\n" NOTICE "\n\n", banner);
 
-    if (rig_init(&rig) == -1)
+    if (rt_global_init() == -1)
+        return -1;
+
+    if (rig_init() == -1)
         return -1;
     rt_init(&rt);
     library_init(&library);
@@ -135,6 +141,7 @@ int main(int argc, char *argv[])
     scanner = DEFAULT_SCANNER;
     timecode = NULL;
     speed = 1.0;
+    protect = false;
 
 #if defined WITH_OSS || WITH_ALSA
     rate = DEFAULT_RATE;
@@ -251,10 +258,9 @@ int main(int argc, char *argv[])
 		  !strcmp(argv[0], "-j"))
 	{
             unsigned int sample_rate;
-            struct deck_t *ld;
+            struct deck *ld;
             struct device *device;
-            struct track_t *track;
-            struct timecoder_t *timecoder;
+            struct timecoder *timecoder;
 
             /* Create a deck */
 
@@ -264,7 +270,7 @@ int main(int argc, char *argv[])
                 return -1;
             }
 
-            if (decks == sizeof deck) {
+            if (decks == ARRAY_SIZE(deck)) {
                 fprintf(stderr, "Too many decks; aborting.\n");
                 return -1;
             }
@@ -274,7 +280,8 @@ int main(int argc, char *argv[])
             ld = &deck[decks];
             device = &ld->device;
             timecoder = &ld->timecoder;
-            track = &ld->track;
+            ld->importer = importer;
+            ld->protect = protect;
 
             /* Work out which device type we are using, and initialise
              * an appropriate device. */
@@ -315,11 +322,10 @@ int main(int argc, char *argv[])
             }
 
             timecoder_init(timecoder, timecode, speed, sample_rate);
-            track_init(track, importer);
 
             /* Connect up the elements to make an operational deck */
 
-            r = deck_init(ld, &rt, &rig);
+            r = deck_init(ld, &rt);
             if (r == -1)
                 return -1;
 
@@ -360,6 +366,20 @@ int main(int argc, char *argv[])
             argv++;
             argc--;
 
+        } else if (!strcmp(argv[0], "-c")) {
+
+            protect = true;
+
+            argv++;
+            argc--;
+
+        } else if (!strcmp(argv[0], "-u")) {
+
+            protect = false;
+
+            argv++;
+            argc--;
+
         } else if (!strcmp(argv[0], "-i")) {
 
             /* Importer script for subsequent decks */
@@ -430,18 +450,18 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-    if (interface_start(&iface, deck, decks, &library, &rig) == -1)
+    if (interface_start(deck, decks, &library) == -1)
         return -1;
     if (rt_start(&rt) == -1)
         return -1;
 
-    if (rig_main(&rig) == -1)
+    if (rig_main() == -1)
         return -1;
 
     fprintf(stderr, "Exiting cleanly...\n");
 
     rt_stop(&rt);
-    interface_stop(&iface);
+    interface_stop();
 
     for (n = 0; n < decks; n++)
         deck_clear(&deck[n]);
@@ -449,7 +469,7 @@ int main(int argc, char *argv[])
     timecoder_free_lookup();
     library_clear(&library);
     rt_clear(&rt);
-    rig_clear(&rig);
+    rig_clear();
 
     fprintf(stderr, "Done.\n");
 
